#!/usr/bin/env perl 

# This script is provided "as is" it is
# not warranted but may be supported on a limited basis
# by Network Appliance.
#
# psf_cp_time.pl
#
#   PerfStatFilter for certain WAFL CP time info. 
#
#   v0.04  01-26-2005 : Added column to output for 'max_CP_msecs'                   
#   v0.03  11-09-2004 : Removed call to psf_versions.pl since values are never used.
#                       Also added -T <timestamp> so we don't have to 
#                       call psf_filer_timestamp.pl 
#   v0.02  03-23-2004 : Added -f <filer> for multi-filer perfstat files.
#                       Only tested against perfstat 5.20 Linux version.
#
#   v0.01  02-25-2004 : Initial implementation.
#

# EXPORTED CONSTANTS (Not really, you have to copy them)

use constant _NA_TIMESTAMP             => 0;
use constant _NA_CP_AVG_MSECS          => 1;
use constant _NA_CP_AVG_BP0_MSECS      => 2;
use constant _NA_CP_AVG_PH0_MSECS      => 3;
use constant _NA_CP_AVG_PH1_MSECS      => 4;
use constant _NA_CP_AVG_QTA_MSECS      => 5;
use constant _NA_CP_AVG_INF_MSECS      => 6;
use constant _NA_CP_AVG_SUM_MSECS      => 7;
use constant _NA_CP_AVG_INO_MSECS      => 8;
use constant _NA_CP_AVG_BM_MSECS       => 9;
use constant _NA_CP_AVG_SNP_MSECS      => 10;
use constant _NA_CP_AVG_FLS_MSECS      => 11;
use constant _NA_CP_AVG_FS_MSECS       => 12;
use constant _NA_CP_AVG_PH4_MSECS      => 13;
use constant _NA_CP_AVG_VOL_MSECS      => 14;
use constant _NA_CP_MAX_MSECS          => 15;
use constant _NA_CP_AVG_PH3_MSECS      => 16; 
use constant _NA_CP_AVG_RVOL_MSECS     => 17;


# Imported
use constant    _NA_VERS_PERFSTAT_MAJOR => 0;
use constant    _NA_VERS_PERFSTAT_MINOR => 1;
use constant    _NA_VERS_ONTAP_VERSION  => 2;
use constant    _NA_VERS_ONTAP_MAJOR    => 3;
use constant    _NA_VERS_ONTAP_MINOR    => 4;
use constant    _NA_VERS_FILER_MODEL    => 5;
use constant    _NA_VERS_FILER_MAGIC1   => 6;
use constant    _NA_VERS_FILER_MAGIC2   => 7;
use constant    _NA_VERS_PERFSTAT_PLATFORM => 8;

# Internal 

use constant FIRST_DATA                => 1;
use constant LAST_DATA                 => 17;


use Getopt::Std;
getopts("dDf:hHnT:");

if ($opt_h) { usage(); }

if ($opt_d) {
   $debug = 1;
} else {
   $debug = 0;
}

if ($opt_D) {
   $data_plus_header = 1;
} else {
   $data_plus_header = 0;
}

if ($opt_f) {
   $filer = $opt_f;
}

if ($opt_H) {
   $header = 1;
} else {
   $header = 0;
}

if ($opt_T) {
   $timestamp  = $opt_T;
} 

if ($opt_n) {
   $nice_output = 1;
} else {
   $nice_output = 0;
}

$cp_avg_msecs, $bp0_msecs, $ph0_msecs, $ph1_msecs, $qta_msecs, $inf_msecs, $sum_msecs;
$ino_msecs, $bm_msecs, $snp_msecs, $fls_msecs, $fs_msecs, $ph4_msecs, $vol_msecs, $cp_max_msecs, $ph3_msecs, $rvol_msecs;

$data[_NA_CP_AVG_MSECS]               = \$cp_avg_msecs;
$data[_NA_CP_AVG_BP0_MSECS]           = \$bp0_msecs;
$data[_NA_CP_AVG_PH0_MSECS]           = \$ph0_msecs;
$data[_NA_CP_AVG_PH1_MSECS]           = \$ph1_msecs;
$data[_NA_CP_AVG_QTA_MSECS]           = \$qta_msecs;
$data[_NA_CP_AVG_INF_MSECS]           = \$inf_msecs;
$data[_NA_CP_AVG_SUM_MSECS]           = \$sum_msecs;
$data[_NA_CP_AVG_INO_MSECS]           = \$ino_msecs;
$data[_NA_CP_AVG_BM_MSECS]            = \$bm_msecs;
$data[_NA_CP_AVG_SNP_MSECS]           = \$snp_msecs;
$data[_NA_CP_AVG_FLS_MSECS]           = \$fls_msecs;
$data[_NA_CP_AVG_FS_MSECS]            = \$fs_msecs;
$data[_NA_CP_AVG_PH4_MSECS]           = \$ph4_msecs;
$data[_NA_CP_AVG_VOL_MSECS]           = \$vol_msecs;
$data[_NA_CP_MAX_MSECS]               = \$cp_max_msecs;
$data[_NA_CP_AVG_PH3_MSECS]           = \$ph3_msecs;
$data[_NA_CP_AVG_RVOL_MSECS]          = \$rvol_msecs;

$header_line = "Timestamp,      avg_CP_msecs, bp0_ms, ph0_ms, ph1_ms, qta_ms, inf_ms, sum_ms, ino_ms, bm_ms, snp_ms, fls_ms, fs_ms, ph4_ms, vol_ms, max_CP_msecs, ph3_ms, rvol_ms";

if ( !$nice_output && $header ) {
   printf ("%s \n", $header_line);
   if ( !$data_plus_header ) {
      exit;
   }
}

$perfstat_file = $ARGV[0];
open(INPUTA, '<' . $perfstat_file) || die 'Unable to open perfstat file.';

$seconds = 0;
$found_wafl_stats = 0;

# Get to POSTSTATs 
while ($Line = <INPUTA>) {
   if ($Line =~ /POSTSTAT/ ) {  
      last ;
   }
}

if (!$timestamp) {
   if ( $filer ) {
      $timestamp = `psf_filer_timestamp.pl -f $filer $perfstat_file`;
   }
   else {
      $timestamp = `psf_filer_timestamp.pl $perfstat_file`;
   }
}
chop $timestamp;

$done = 0;
while ($Line = <INPUTA>) {
   if ( $done ) {
      last;
   }
   if ( ( $Line =~ /^=-=-/ ) &&
           ( $Line =~ /POSTSTATS/ ) &&
           ( $Line =~ /wafl_susp -w/ ) ) {
      @line = split (/\s+/, $Line);
      $curr_filer = $line[2];
      if ( $debug ) {
         printf ("Current statit filer : %s \n", $curr_filer);
      }
   }
   elsif ( ($Line =~ /WAFL statistics over / )  &&
           ( (!$filer) ||
             ($filer =~ /^$curr_filer/) ) ) {
      if ( $debug ) { print "Found wafl_susp \n"; }
      $found_wafl_stats = 1;
      @line = split (/ +/, $Line);
      $seconds = $line[3];
      $num_cps = 0;
      while ($Line = <INPUTA>) { 
               if ($Line =~ /POSTSTAT/ ) {
                  $done = 1;
                  last;
               }
               if ($Line =~ /^cp_from_timer/ ){
                  @line = split (/ +/, $Line);
                  $cp_from_timer = $line[2];
                  $num_cps += $cp_from_timer;
                  if ($debug) {printf ("cp_from_timer : %d \n", $cp_from_timer);}
                  next;
               }
               elsif ($Line =~ /^cp_from_snapshot/ ){
                  @line = split (/ +/, $Line);
                  $cp_from_snapshot = $line[2];
                  $num_cps += $cp_from_snapshot;
                  if ($debug) {printf ("cp_from_snapshot : %d \n", $cp_from_snapshot);}
                  next;
               }
               elsif ($Line =~ /^cp_from_low_water/ ){
                  @line = split (/ +/, $Line);
                  $cp_from_low_water = $line[2];
                  $num_cps += $cp_from_low_water;
                  if ($debug) {printf ("cp_from_low_water: %d \n", $cp_from_low_water);}
                  next;
               }
               elsif ($Line =~ /^cp_from_high_water/ ){
                  @line = split (/ +/, $Line);
                  $cp_from_high_water = $line[2];
                  $num_cps += $cp_from_high_water;
                  if ($debug) {printf ("cp_from_high_water: %d \n", $cp_from_high_water);}
                  next;
               }
               elsif ($Line =~ /^cp_from_log_full/ ){
                  @line = split (/ +/, $Line);
                  $cp_from_log_full = $line[2];
                  $num_cps += $cp_from_log_full;
                  if ($debug) {printf ("cp_from_log_full: %d \n", $cp_from_log_full);}
                  next;
               }
               elsif ($Line =~ /^cp_from_cp/ ){
                  @line = split (/ +/, $Line);
                  $cp_from_cp = $line[2];
                  $num_cps += $cp_from_cp;
                  if ($debug) {printf ("cp_from_cp : %d \n", $cp_from_cp );}
                  next;
               }
               elsif ($Line =~ /^cp_from_flush/ ){
                  @line = split (/ +/, $Line);
                  $cp_from_flush = $line[2];
                  $num_cps += $cp_from_flush;
                  if ($debug) {printf ("cp_from_flush : %d \n", $cp_from_flush );}
                  next;
               }
               elsif ($Line =~ /^cp_from_sync/ ){
                  @line = split (/ +/, $Line);
                  $cp_from_sync = $line[2];
                  $num_cps += $cp_from_sync;
                  if ($debug) {printf ("cp_from_sync : %d \n", $cp_from_sync );}
                  next;
               }
               elsif ($Line =~ /^cp_from_vbuf/ ){
                  @line = split (/ +/, $Line);
                  $cp_from_vbuf = $line[2];
                  $num_cps += $cp_from_vbuf;
                  if ($debug) {printf ("cp_from_vbuf : %d \n", $cp_from_vbuf );}
                  next;
               }
               elsif ($Line =~ /^cp_from/ ){
                  @line = split (/ +/, $Line);
                  $num_cps += $line[2];
                  if ($debug) {printf ("cp_from other : %d \n", $line[2] );}
                  next;
               }

               elsif ($Line =~ /^cp times/ ){
		  for ($k = 1; $k < 6; $k++) { 
                  	@line = split (/ +/, $Line);
			if ($k < 2) { 
                  		$cp_avg_msecs = $line[3]/$num_cps;
			}
			parse_cp_times_line (@line);
                        $Line = <INPUTA>;
		  }
                  next;
               }
               elsif ($Line =~ /^max_cp_time/ ) {
                  @line = split (/ +/, $Line);
                  $cp_max_msecs = $line[2];
                  next;
               }
      }
   }
}
if ( $found_wafl_stats ) {

   if ( $nice_output ) {
      printf ("Average msecs per CP                    : \t %.1f \n", $cp_avg_msecs);
      printf ("Max     msecs per CP                    : \t %.1f \n", $cp_max_msecs);
      printf ("Avg msecs in bp0  (resync topology)     : \t %.1f \n", $bp0_msecs);
      printf ("Avg msecs in ph0  (snap deletes)        : \t %.1f \n", $ph0_msecs);
      printf ("Avg msecs in ph1  (write-alloc)          : \t %.1f \n", $ph1_msecs);
      printf ("Avg msecs in qta  (phase 2 quota file)  : \t %.1f \n", $qta_msecs);
      printf ("Avg msecs in inf  (phase 2 dirty inode) : \t %.1f \n", $inf_msecs);
      printf ("Avg msecs in sum  (phase 2 summary file): \t %.1f \n", $sum_msecs);
      printf ("Avg msecs in ino  (write-alloc inode)   : \t %.1f \n", $ino_msecs);
      printf ("Avg msecs in bm   (write-alloc blockmap): \t %.1f \n", $bm_msecs);
      printf ("Avg msecs in snp  (cre/del snap inodes) : \t %.1f \n", $snp_msecs);
      printf ("Avg msecs in fls  (flush dirty stripes) : \t %.1f \n", $fls_msecs);
      printf ("Avg msecs in fs   (sync fsinfo)         : \t %.1f \n", $fs_msecs);
      printf ("Avg msecs in ph4  (phase4)              : \t %.1f \n", $ph4_msecs);
      printf ("Avg msecs in vol  (sync volinfo)        : \t %.1f \n", $vol_msecs);
   }
   else {
      printf ("%s, ", $timestamp);
      for ( $i=FIRST_DATA; $i <= LAST_DATA; $i++ ) {
         printf ("%.1f,    ", ${$data[$i]} );
      }
      printf ("\n");
   }
}

sub parse_cp_times_line {
   local (@line) =  @_ ;
   my $i;

   for ($i=0; $i < scalar(@line); $i++) {
	if ($line[$i] eq "bp0") {
		$bp0_msecs = $line[$i+1]/$num_cps;
	} 
	elsif ($line[$i] eq "ph0") {
		$ph0_msecs = $line[$i+1]/$num_cps;
	} 
	elsif ($line[$i] eq "ph1") {
		$ph1_msecs = $line[$i+1]/$num_cps;
	} 
	elsif ($line[$i] eq "qta") {
		$qta_msecs = $line[$i+1]/$num_cps;
	} 
	elsif ($line[$i] eq "inf") {
		$inf_msecs = $line[$i+1]/$num_cps;
	} 
	elsif ($line[$i] eq "sum") {
		$sum_msecs = $line[$i+1]/$num_cps;
	} 
	elsif ($line[$i] eq "ino") {
		$ino_msecs = $line[$i+1]/$num_cps;
	} 
	elsif ($line[$i] eq "bm") {
		$bm_msecs = $line[$i+1]/$num_cps;
	} 
	elsif ($line[$i] eq "snp") {
		$snp_msecs = $line[$i+1]/$num_cps;
	} 
	elsif ($line[$i] eq "fls") {
		$fls_msecs = $line[$i+1]/$num_cps;
	} 
	elsif ($line[$i] eq "fs") {
		$fs_msecs = $line[$i+1]/$num_cps;
	} 
	elsif ($line[$i] eq "vol") {
		$vol_msecs = $line[$i+1]/$num_cps;
	} 
	elsif ($line[$i] eq "ph3") {
		$ph3_msecs = $line[$i+1]/$num_cps;
	} 
	elsif ($line[$i] eq "ph4") {
		$ph4_msecs = $line[$i+1]/$num_cps;
	} 
	elsif ($line[$i] eq "rvol") {
		$rvol_msecs = $line[$i+1]/$num_cps;
	}
   } 
}

sub usage {
   print <<EOM;
psf_cp_times.pl [-dDHhn] [-f <filer>] <perfstat filename>

Summarizes the wafl cp timing stats data from perfstat outpt files.

    Arguments and switches -
       -d          Print debug messages.
       -D          Print data as well as a header. Only relevent if -H specified. 
       -f <filer>  For multi-filer perfstats choose this one.  
       -H          Print a header line only unless -D specified. 
       -h          This message.
       -n          Print Nice output, a value per line labeled with what it is.
EOM
exit;

}

