#!/usr/bin/env perl 

# This script is provided "as is" it is
# not warranted but may be supported on a limited basis
# by Network Appliance.
#
# psf_get_timerange.pl
#
#   PerfStatFilter obtaining a start time and end time of multi-iteration
#	perfstat file.
#

use Time::Local;

use constant  SEEK_SET 	=> 0;
use constant  SEEK_CUR 	=> 1;
use constant  SEEK_END 	=> 2;

use Getopt::Std;
getopts("df:");

if ($opt_h) { usage(); }

if ($opt_f) {
   $file_root = $opt_f;; 
} 

if ($opt_d) {
   $debug = 1; 
} else {
   $debug = 0;
}

# To make this reasonably fast, try to avoid reading every line. 
#    First get the filesize in bytes
#    Also get the number of iterations
#    On each iteration do a seek to start scanning closer to where you 
#          expect a new iteration. 

$perfstat_file = $ARGV[0];
open(INPUTA, '<' . $perfstat_file) || die 'Unable to open perfstat file (arg1) .';

seek (INPUTA, -1, SEEK_END );
$filesize = tell (INPUTA);
$num_iterations = get_num_iterations ($perfstat_file);
if ($debug) {
	printf ("Filesize: %d, Num Iterations: %d,  \n",
		$filesize, $num_iterations);
}

seek (INPUTA, 0, SEEK_SET );

while (!eof (INPUTA) ) {
        $Line = <INPUTA>;
        if ($Line =~ / BEGIN Iteration/) {
                if ($debug) {
                        printf ("%s ", $Line);
                }
                @line = split (/\s+/, $Line);
                $iteration = $line[3];
		if ($iteration ==  1)  { 
                        $pos_of_first_iter = tell (INPUTA);
			$timestamp = get_statit_starttime();
			$dayofweek = timestamp_to_dayofweek ($timestamp);
                	printf ("%s, %s, %d, \n",
				$dayofweek,
                                $timestamp,
                                $iteration);
                } elsif ($iteration == 2 ) { 
                        $pos_of_second_iter = tell (INPUTA); 
                        $iteration_size = $pos_of_second_iter - $pos_of_first_iter;
                        if ($debug) {
                                printf ("Iteration Size : %d \n", $iteration_size);
                        }
                        last;
                }
	}
}

# Look for last iteration starting at $filesize - $pos_of_first_iter
$pos = $filesize - $iteration_size;
seek (INPUTA, $pos, SEEK_SET);
while (!eof (INPUTA) ) { 
	$Line = <INPUTA>;
	if ($Line =~ / BEGIN Iteration/) {
		if ($debug) { 
			printf ("%s ", $Line);
		}
		@line = split (/\s+/, $Line);
		$iteration = $line[3];
		$timestamp = get_statit_starttime();
	}
}
$dayofweek = timestamp_to_dayofweek ($timestamp);
printf ("%s, %s, %d, \n",
	   $dayofweek,
           $timestamp,
           $iteration);

sub get_statit_starttime {

	my $Line, @line, $result;
	my $mo, $mo_num, $day, $yr, $time;

	while ($Line = <INPUTA>) {
   		if ($Line =~ / Start time:/ ) {
      			@line = split (/ +/, $Line);
      			$mo = $line[4];
      			$day = $line[5];
      			$time = $line[6];
      			$yr = $line[8];
			$mo_num = month_to_numeric ($mo);
             		$result = sprintf ("%d\/%d\/%d %s",
                        		$mo_num, $day,
                        		$yr, $time);
			last;
		}
	}
	return ($result);
}

sub month_to_numeric {
   local ($month) = @_ ;
   my $result;

   if ($month =~ /Jan/ ) {
        $result = 1;
   }
   elsif ($month =~ /Feb/ ) {
        $result = 2;
   }
   elsif ($month =~ /Mar/ ) {
        $result = 3;
   }
   elsif ($month =~ /Apr/ ) {
        $result = 4;
   }
   elsif ($month =~ /May/ ) {
        $result = 5;
   }
   elsif ($month =~ /Jun/ ) {
        $result = 6;
   }
   elsif ($month =~ /Jul/ ) {
        $result = 7;
   }
   elsif ($month =~ /Aug/ ) {
        $result = 8;
   }
   elsif ($month =~ /Sep/ ) {
        $result = 9;
   }
   elsif ($month =~ /Oct/ ) {
        $result = 10;
   }
   elsif ($month =~ /Nov/ ) {
        $result = 11;
   }
   elsif ($month =~ /Dec/ ) {
        $result = 12;
   }
   else {
        $result = -1;
   }
   return $result;
}

sub timestamp_to_dayofweek {
   my ($ts) = @_;

   my (@timestamp_parts, $date, @dateparts);
   my ($mo, $day, $year);
   my ($time, @timeparts);
   my ($hour, $min, $sec);
   my ($dayofweek);
   my ($day_name);

   @timestamp_parts = split (/ /, $ts);
   $date = $timestamp_parts[0];
   $time = $timestamp_parts[1];
   @dateparts = split (/\//, $date);
   @timeparts = split (/:/, $time);
   $mo = $dateparts[0] - 1;   # Month for timegm ranges from 0..11 
   $day = $dateparts[1];
   $year = $dateparts[2];
   $hour = $timeparts[0];
   $min  = $timeparts[1];
   $sec  = $timeparts[2];

   $time = timegm ($sec, $min, $hour, $day, $mo, $year);
   ( $dayofweek )  = ( gmtime ($time) ) [6];
   $day_name = ('Sunday','Monday','Tuesday','Wednesday','Thursday','Friday','Saturday')[$dayofweek];

   return ($day_name);
}

sub get_num_iterations {
        local ($file) = @_;
        my $Line, @line, $iterations, $len;

        $iterations = 1;
        open(LOCINPUTA, '<' . $file) || die 'Unable to open perfstat file';
        while ($Line = <LOCINPUTA>) {
                if ($Line =~ /^=-=-=-=-=-/) {
                        last;
                }
                elsif ($Line =~ /^ITERATIONS/) {
                        $Line =~ s/\r//g;     # Remove DOS carraige returns
                        @line = split (/\s+/, $Line);
                        $iterations = $line[2];
                        # Remove quotes
                        $len = length $iterations;
                        if ($iterations =~ /\"/) {
                                $iterations = substr ($iterations, 1, $len-1);
                                $len--;
                        }
                        if ($iterations =~ /\"/) {
                                $iterations = substr ($iterations, 0, $len-1);
                        }
                        if ($debug) { printf ("iterations: %d \n", $iterations); }
                        last;
                }
                else {
                        next;
                }
        }
        return $iterations;
}
